package cn.nosum.support.proxy;

/**
 * Target Object Information
 *
 * @author Young.
 */
public interface Target<T> {

    /**
     * Dynamic Client Name.
     *
     * @return name
     */
    String name();

    /**
     * Dynamic Client Type
     *
     * @return type
     */
    Class<T> type();

    /**
     * Dynamic Client RefName.
     *
     * @return refName
     */
    String refName();

    /**
     * Dynamic Client RefType.
     *
     * @return refType
     */
    Class<?> refType();


    class DefaultTarget<T> implements Target<T> {
        private final Class<T> type;
        private final String name;

        private final String refName;
        private final Class<?> refType;

        public DefaultTarget(String name, Class<T> type, String refName, Class<?> refType) {
            this.name = name;
            this.type = type;
            this.refName = refName;
            this.refType = refType;
        }

        @Override
        public Class<T> type() {
            return type;
        }

        @Override
        public String name() {
            return name;
        }

        @Override
        public String refName() {
            return refName;
        }

        @Override
        public Class<?> refType() {
            return refType;
        }
    }

}
