package cn.nosum.support.chain;

import org.springframework.util.StopWatch;

import java.util.List;

/**
 * 提供对节点操作的处理器.
 *
 * @author Young
 */
public interface Processor {

  /**
   * 返回执行节点日志
   *
   * @return 执行的节点信息
   */
  List<String> getNodeLog();

  /**
   * 获取StopWatch
   *
   * @return {@link StopWatch} stop watch
   */
  StopWatch getStopWatch();

  /**
   * 输入执行的节点信息
   *
   * @return 正在执行的节点信息
   */
  String prettyPrint();

  /**
   * 清除节点日志
   */
  void clearLog();

  /**
   * 记录链路执行节点日志.
   *
   * @param chainName 节点日志
   */
  void addNodeLog(String chainName);

  /**
   * 设置是否执行下一个节点.
   *
   * @param executeNextNode 是否执行下一个节点
   */
  void setExecuteNextNode(boolean executeNextNode);
}
