package cn.nosum.http;

import cn.nosum.http.exception.HttpErrorException;

import java.io.IOException;

/**
 * http请求执行器.
 *
 * @param <T> 请求参数类型
 * @param <R> 返回值类型
 * @author Young
 */
public interface RequestExecutor<T, R> {

    /**
     * 执行http请求.
     *
     * @param uri  uri
     * @param data 数据
     * @return 响应结果
     * @throws HttpErrorException 自定义异常
     * @throws IOException        io异常
     */
    R execute(String uri, T data) throws HttpErrorException, IOException;

    /**
     * 执行http请求.
     *
     * @param uri     uri
     * @param data    数据
     * @param handler http响应处理器
     * @return handler 处理后的内容
     * @throws HttpErrorException 自定义异常
     * @throws IOException        io异常
     */
    <V> V execute(String uri, T data, ResponseHandler<R,V> handler) throws HttpErrorException, IOException;
}
