package cn.nosum.http;

import cn.nosum.http.apache.ApacheDownloadRequestExecutor;
import cn.nosum.http.apache.ApacheSimpleGetRequestExecutor;
import cn.nosum.http.entity.DownloadResponse;
import cn.nosum.http.exception.HttpErrorException;
import cn.nosum.http.jodd.JoddDownloadRequestExecutor;
import cn.nosum.http.jodd.JoddHttpSimpleGetRequestExecutor;
import cn.nosum.http.okhttp.OkHttpDownloadRequestExecutor;
import cn.nosum.http.okhttp.OkHttpSimpleGetRequestExecutor;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


/**
 * 简单文件下载抽象实现.
 *
 * @param <H> 请求对象
 * @param <P> 代理对象
 * @author Young
 */
public abstract class SimpleDownloadRequestExecutor<H, P>  extends SimpleRequestExecutor<H, P, String, DownloadResponse> {

    public SimpleDownloadRequestExecutor(RequestHttp<H, P> requestHttp) {
        super(requestHttp);
    }

    public SimpleDownloadRequestExecutor(RequestHttp<H, P> requestHttp, ResponseHandler<DownloadResponse,DownloadResponse> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public <V> V execute(String uri, String queryParam, ResponseHandler<DownloadResponse,V> handler) throws HttpErrorException, IOException {
        if (queryParam != null) {
            if (uri.indexOf('?') == -1) {
                uri += '?';
            }
            uri += uri.endsWith("?") ? queryParam : '&' + queryParam;
        }
        return handler.handle(doExecute(uri, queryParam));
    }

    public static RequestExecutor<String, DownloadResponse> create(RequestHttp requestHttp) {
        switch (requestHttp.getRequestType()) {
            case APACHE_HTTP:
                return new ApacheDownloadRequestExecutor(requestHttp);
            case JODD_HTTP:
                return new JoddDownloadRequestExecutor(requestHttp);
            case OK_HTTP:
                return new OkHttpDownloadRequestExecutor(requestHttp);
            default:
                throw new IllegalArgumentException("非法请求参数");
        }
    }


}
