package cn.nosum.http;

import cn.nosum.http.apache.ApacheSimplePostRequestExecutor;
import cn.nosum.http.exception.HttpError;
import cn.nosum.http.exception.HttpErrorException;
import cn.nosum.http.jodd.JoddHttpSimplePostRequestExecutor;
import cn.nosum.http.okhttp.OkHttpSimplePostRequestExecutor;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;

/**
 * 简单的POST请求执行器.
 *
 * @param <H> 请求对象
 * @param <P> 代理对象
 * @param <R> 返回类型
 * @author Young
 */
public abstract class SimplePostRequestExecutor<H, P, R> extends SimpleRequestExecutor<H, P, String, R> {

    public SimplePostRequestExecutor(RequestHttp<H, P> requestHttp) {
        super(requestHttp);
    }

    public SimplePostRequestExecutor(RequestHttp<H, P> requestHttp, ResponseHandler<R,R> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public <V> V execute(String uri, String data, ResponseHandler<R,V> handler) throws HttpErrorException, IOException {
        return handler.handle(doExecute(uri, data));
    }

    public static RequestExecutor<String, String> create(RequestHttp requestHttp) {
        switch (requestHttp.getRequestType()) {
            case APACHE_HTTP:
                return new ApacheSimplePostRequestExecutor(requestHttp);
            case JODD_HTTP:
                return new JoddHttpSimplePostRequestExecutor(requestHttp);
            case OK_HTTP:
                return new OkHttpSimplePostRequestExecutor(requestHttp);
            default:
                throw new IllegalArgumentException("非法请求参数");
        }
    }

}
