package cn.nosum.http;

import cn.nosum.http.exception.HttpErrorException;

import java.io.IOException;

/**
 * 简单的请求执行器.
 *
 * @param <H> 请求对象
 * @param <P> 代理对象
 * @param <T> 参数类型
 * @param <R> 返回类型
 * @author Young
 */
public abstract class SimpleRequestExecutor<H, P, T, R> implements RequestExecutor<T, R> {


    protected RequestHttp<H, P> requestHttp;

    protected ResponseHandler<R, R> responseHandler;

    public SimpleRequestExecutor(RequestHttp<H, P> requestHttp) {
        this.requestHttp = requestHttp;
        this.responseHandler = responseContent -> responseContent;
    }

    public SimpleRequestExecutor(RequestHttp<H, P> requestHttp, ResponseHandler<R, R> responseHandler) {
        this.requestHttp = requestHttp;
        this.responseHandler = responseHandler;
    }

    @Override
    public R execute(String uri, T data) throws HttpErrorException, IOException {
        return execute(uri, data, responseHandler);
    }

    /**
     * 设置响应处理器.
     *
     * @param responseHandler 响应处理器
     */
    public void setResponseHandler(ResponseHandler<R, R> responseHandler) {
        this.responseHandler = responseHandler;
    }

    /**
     * 执行http请求.
     *
     * @param uri  uri
     * @param data 数据
     * @return 响应结果
     * @throws HttpErrorException 自定义异常
     * @throws IOException        io异常
     */
    public abstract R doExecute(String uri, T data) throws HttpErrorException, IOException;

}
