package cn.nosum.http;

import cn.nosum.http.apache.ApacheSimpleUploadRequestExecutor;
import cn.nosum.http.entity.UploadRequest;
import cn.nosum.http.exception.HttpErrorException;
import cn.nosum.http.jodd.JoddHttpSimpleUploadRequestExecutor;
import cn.nosum.http.okhttp.OkHttpSimpleUploadRequestExecutor;
import org.apache.commons.lang3.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Objects;
import java.util.UUID;

/**
 * 文件上传请求执行器
 *
 * @param <H> 请求对象
 * @param <P> 代理对象
 * @param <R> 返回类型
 * @author Young
 */
public abstract class SimpleUploadRequestExecutor<H, P, R> extends SimpleRequestExecutor<H, P, UploadRequest, R> {

    public SimpleUploadRequestExecutor(RequestHttp<H, P> requestHttp) {
        super(requestHttp);
    }

    public SimpleUploadRequestExecutor(RequestHttp<H, P> requestHttp, ResponseHandler<R,R> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public <V> V execute(String uri, UploadRequest uploadRequest, ResponseHandler<R,V> handler) throws HttpErrorException, IOException {
        if (Objects.isNull(uploadRequest) || Objects.isNull(uploadRequest.getInputStream())) {
            throw new HttpErrorException("request file cannot be empty");
        }
        if (StringUtils.isBlank(uploadRequest.getName())) {
            uploadRequest.setName("media");
        }
        if (StringUtils.isBlank(uploadRequest.getFileName())){
            uploadRequest.setFileName(UUID.randomUUID().toString().replaceAll("-",""));
        }
        return handler.handle(doExecute(uri, uploadRequest));
    }

    protected byte[] transfer(InputStream inputStream) throws IOException {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int n;
        while (-1 != (n = inputStream.read(buffer))) {
            output.write(buffer, 0, n);
        }
        return output.toByteArray();
    }

    public static RequestExecutor<UploadRequest, String> create(RequestHttp requestHttp) {
        switch (requestHttp.getRequestType()) {
            case APACHE_HTTP:
                return new ApacheSimpleUploadRequestExecutor(requestHttp);
            case JODD_HTTP:
                return new JoddHttpSimpleUploadRequestExecutor(requestHttp);
            case OK_HTTP:
                return new OkHttpSimpleUploadRequestExecutor(requestHttp);
            default:
                return null;
        }
    }
}
