package cn.nosum.http.apache;

import cn.nosum.http.RequestHttp;
import cn.nosum.http.ResponseHandler;
import cn.nosum.http.SimpleDownloadRequestExecutor;
import cn.nosum.http.entity.DownloadResponse;
import cn.nosum.http.exception.HttpErrorException;
import cn.nosum.http.proxy.HttpResponseProxy;
import org.apache.http.HttpHost;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;

import java.io.IOException;


/**
 * APACHE 下载请求封装.
 *
 * @author Young
 */
public class ApacheDownloadRequestExecutor extends SimpleDownloadRequestExecutor<CloseableHttpClient, HttpHost> {


    public ApacheDownloadRequestExecutor(RequestHttp<CloseableHttpClient, HttpHost> requestHttp) {
        super(requestHttp);
    }

    public ApacheDownloadRequestExecutor(RequestHttp<CloseableHttpClient, HttpHost> requestHttp, ResponseHandler<DownloadResponse, DownloadResponse> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public DownloadResponse doExecute(String uri, String data) throws HttpErrorException, IOException {
        HttpGet httpGet = new HttpGet(uri);
        if (requestHttp.getRequestHttpProxy() != null) {
            RequestConfig config = RequestConfig.custom().setProxy(requestHttp.getRequestHttpProxy()).build();
            httpGet.setConfig(config);
        }

        try (CloseableHttpResponse response = requestHttp.getRequestHttpClient().execute(httpGet)) {
            return new HttpResponseProxy(response).transferDownload();
        } finally {
            httpGet.releaseConnection();
        }
    }
}
