package cn.nosum.http.apache;

import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;

/**
 * httpclient build interface.
 *
 * @author Young
 */
public interface ApacheHttpClientBuilder {

    /**
     * 构建httpclient实例.
     *
     * @return new instance of CloseableHttpClient
     */
    CloseableHttpClient build();

    /**
     * 代理服务器地址.
     *
     * @param httpProxyHost HTTP 代理对象
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder httpProxyHost(String httpProxyHost);

    /**
     * 代理服务器端口.
     *
     * @param httpProxyPort 代理服务器端口
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder httpProxyPort(int httpProxyPort);

    /**
     * 代理服务器用户名.
     *
     * @param httpProxyUsername 代理服务器用户名
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder httpProxyUsername(String httpProxyUsername);

    /**
     * 代理服务器密码.
     *
     * @param httpProxyPassword 代理服务器密码
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder httpProxyPassword(String httpProxyPassword);

    /**
     * 连接超时时间.
     *
     * @param connTimeout 连接超时时间
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder connectTimeout(Integer connTimeout);

    /**
     * 请求超时时间.
     *
     * @param readTimeout 请求超时时间
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder readTimeout(Integer readTimeout);

    /**
     * ssl连接socket工厂.
     *
     * @param sslConnectionSocketFactory SSL 连接工厂
     * @return httpclient build interface.
     */
    ApacheHttpClientBuilder sslConnectionSocketFactory(SSLConnectionSocketFactory sslConnectionSocketFactory);
}
