package cn.nosum.http.apache;

import cn.nosum.http.RequestHttp;
import cn.nosum.http.ResponseHandler;
import cn.nosum.http.SimpleUploadRequestExecutor;
import cn.nosum.http.entity.UploadRequest;
import cn.nosum.http.exception.HttpErrorException;
import org.apache.commons.lang3.RandomUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;

import java.io.IOException;
import java.util.UUID;


/**
 * APACHE 简单文件上传实现.
 *
 * @author Young
 */
public class ApacheSimpleUploadRequestExecutor extends SimpleUploadRequestExecutor<CloseableHttpClient, HttpHost, String> {

    public ApacheSimpleUploadRequestExecutor(RequestHttp<CloseableHttpClient, HttpHost> requestHttp) {
        super(requestHttp);
    }

    public ApacheSimpleUploadRequestExecutor(RequestHttp<CloseableHttpClient, HttpHost> requestHttp, ResponseHandler<String,String> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public String doExecute(String uri, UploadRequest uploadRequest) throws HttpErrorException, IOException {
        HttpPost httpPost = new HttpPost(uri);
        if (requestHttp.getRequestHttpProxy() != null) {
            RequestConfig config = RequestConfig.custom().setProxy(requestHttp.getRequestHttpProxy()).build();
            httpPost.setConfig(config);
        }
        HttpEntity entity = MultipartEntityBuilder
                .create()
                .addBinaryBody(uploadRequest.getName(), uploadRequest.getInputStream(), ContentType.DEFAULT_BINARY, uploadRequest.getName())
                .setMode(HttpMultipartMode.RFC6532)
                .build();
        httpPost.setEntity(entity);
        try (CloseableHttpResponse response = requestHttp.getRequestHttpClient().execute(httpPost)) {
            return Utf8ResponseHandler.INSTANCE.handleResponse(response);
        } finally {
            httpPost.releaseConnection();
        }
    }
}
