package cn.nosum.http.okhttp;

import cn.nosum.http.enums.ProxyType;

import java.net.InetSocketAddress;
import java.net.Proxy;

/**
 * OK HTTP 代理对象实体.
 *
 * @author Young
 */
public class OkHttpProxyInfo {

    /**
     * 代理服务器地址
     */
    private final String proxyAddress;
    /**
     * 代理服务器端口
     */
    private final int proxyPort;

    /**
     * 代理用户名称
     */
    private final String proxyUsername;

    /**
     * 代理密码
     */
    private final String proxyPassword;

    /**
     * 代理类型
     */
    private final ProxyType proxyType;

    public OkHttpProxyInfo(ProxyType proxyType, String proxyHost, int proxyPort, String proxyUser, String proxyPassword) {
        this.proxyType = proxyType;
        this.proxyAddress = proxyHost;
        this.proxyPort = proxyPort;
        this.proxyUsername = proxyUser;
        this.proxyPassword = proxyPassword;
    }

    /**
     * Creates directProxy.
     */
    public static OkHttpProxyInfo directProxy() {
        return new OkHttpProxyInfo(ProxyType.NONE, null, 0, null, null);
    }


    /**
     * Creates SOCKS4 proxy.
     */
    public static OkHttpProxyInfo socks4Proxy(String proxyAddress, int proxyPort, String proxyUser) {
        return new OkHttpProxyInfo(ProxyType.SOCKS4, proxyAddress, proxyPort, proxyUser, null);
    }

    /**
     * Creates SOCKS5 proxy.
     */
    public static OkHttpProxyInfo socks5Proxy(String proxyAddress, int proxyPort, String proxyUser, String proxyPassword) {
        return new OkHttpProxyInfo(ProxyType.SOCKS5, proxyAddress, proxyPort, proxyUser, proxyPassword);
    }

    /**
     * Creates HTTP proxy.
     */
    public static OkHttpProxyInfo httpProxy(String proxyAddress, int proxyPort, String proxyUser, String proxyPassword) {
        return new OkHttpProxyInfo(ProxyType.HTTP, proxyAddress, proxyPort, proxyUser, proxyPassword);
    }

    /**
     * Returns proxy type.
     */
    public ProxyType getProxyType() {
        return proxyType;
    }


    /**
     * Returns proxy address.
     */
    public String getProxyAddress() {
        return proxyAddress;
    }

    /**
     * Returns proxy port.
     */
    public int getProxyPort() {
        return proxyPort;
    }

    /**
     * Returns proxy userName or <code>null</code> if
     * no authentication required.
     */
    public String getProxyUsername() {
        return proxyUsername;
    }

    /**
     * Returns proxy password or <code>null</code>.
     */
    public String getProxyPassword() {
        return proxyPassword;
    }

    /**
     * 返回 java.net.Proxy
     */
    public Proxy getProxy() {
        Proxy proxy = null;
        if (getProxyType().equals(ProxyType.SOCKS5)) {
            proxy = new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(getProxyAddress(), getProxyPort()));
        } else if (getProxyType().equals(ProxyType.SOCKS4)) {
            proxy = new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(getProxyAddress(), getProxyPort()));
        } else if (getProxyType().equals(ProxyType.HTTP)) {
            proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(getProxyAddress(), getProxyPort()));
        } else if (getProxyType().equals(ProxyType.NONE)) {
            proxy = new Proxy(Proxy.Type.DIRECT, new InetSocketAddress(getProxyAddress(), getProxyPort()));
        }
        return proxy;
    }


}
