package cn.nosum.http.okhttp;


import cn.nosum.http.RequestHttp;
import cn.nosum.http.ResponseHandler;
import cn.nosum.http.SimpleUploadRequestExecutor;
import cn.nosum.http.entity.UploadRequest;
import cn.nosum.http.exception.HttpErrorException;
import okhttp3.*;

import java.io.IOException;
import java.util.Objects;

/**
 * OKHTTP 简单文件上传实现类.
 *
 * @author Young
 */
public class OkHttpSimpleUploadRequestExecutor extends SimpleUploadRequestExecutor<OkHttpClient, OkHttpProxyInfo, String> {


    public OkHttpSimpleUploadRequestExecutor(RequestHttp<OkHttpClient, OkHttpProxyInfo> requestHttp) {
        super(requestHttp);
    }

    public OkHttpSimpleUploadRequestExecutor(RequestHttp<OkHttpClient, OkHttpProxyInfo> requestHttp, ResponseHandler<String, String> responseHandler) {
        super(requestHttp, responseHandler);
    }

    @Override
    public String doExecute(String uri, UploadRequest uploadRequest) throws HttpErrorException, IOException {
        RequestBody body = new MultipartBody.Builder()
                .setType(Objects.requireNonNull(MediaType.parse("multipart/form-data")))
                .addFormDataPart(uploadRequest.getName(),
                        uploadRequest.getName(),
                        RequestBody.create(MediaType.parse("application/octet-stream"), transfer(uploadRequest.getInputStream())))
                .build();

        Request request = new Request.Builder().url(uri).post(body).build();

        Response response = requestHttp.getRequestHttpClient().newCall(request).execute();
        return Objects.requireNonNull(response.body()).string();
    }
}
