package cn.nosum.wx.common.error;

import cn.nosum.http.exception.HttpError;
import cn.nosum.wx.common.utils.json.WxGsonBuilder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.StringUtils;

/**
 * 微信错误码.
 * 请阅读：
 * 公众平台：http://mp.weixin.qq.com/wiki/10/6380dc743053a91c544ffd2b7c959166.html
 * 企业微信：https://work.weixin.qq.com/api/doc#10649
 *
 * @author Young
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class WxError extends HttpError {
    private static final long serialVersionUID = 7869786563361406291L;

    private String json;

    public static WxError fromJson(String json) {
        final WxError wxError = WxGsonBuilder.create().fromJson(json, WxError.class);
        if (wxError.getErrorCode() == 0) {
            return wxError;
        }

        if (StringUtils.isNotEmpty(wxError.getErrorMsg())) {
            wxError.setErrorMsg(wxError.getErrorMsg());
        }

        final String msg = WxCpErrorMsgEnum.findMsgByCode(wxError.getErrorCode());
        if (msg != null) {
            wxError.setErrorMsg(msg);
        }

        return wxError;
    }

    @Override
    public String toString() {
        if (this.json == null) {
            return "错误代码：" + this.getErrorCode() + ", 错误信息：" + this.getErrorMsg();
        }
        return "错误代码：" + this.getErrorCode() + ", 错误信息：" + this.getErrorMsg() + "，微信原始报文：" + this.json;
    }

}
