package cn.nosum.wx.common.service;

import cn.nosum.wx.common.entity.ToJson;
import cn.nosum.wx.common.error.WxErrorException;
import com.google.gson.JsonObject;

/**
 * 微信服务接口.
 *
 * @author Young
 */
public interface WxService {
    /**
     * 当本Service没有实现某个API的时候，可以用这个，针对所有微信API中的GET请求.
     *
     * @param queryParam 参数
     * @param url        请求接口地址
     * @return 接口响应字符串
     */
    String get(String url, String queryParam);

    /**
     * 当本Service没有实现某个API的时候，可以用这个，针对所有微信API中的POST请求.
     *
     * @param postData 请求参数json值
     * @param url      请求接口地址
     * @return 接口响应字符串
     */
    String post(String url, String postData);

    /**
     * 当本Service没有实现某个API的时候，可以用这个，针对所有微信API中的POST请求.
     *
     * @param url 请求接口地址
     * @param obj 请求对象
     * @return 接口响应字符串
     * @throws WxErrorException 异常
     */
    String post(String url, Object obj) ;

    /**
     * 当本Service没有实现某个API的时候，可以用这个，针对所有微信API中的POST请求.
     *
     * @param url        请求接口地址
     * @param jsonObject 请求对象
     * @return 接口响应字符串
     * @throws WxErrorException 异常
     */
    String post(String url, JsonObject jsonObject) ;

    /**
     * 当本Service没有实现某个API的时候，可以用这个，针对所有微信API中的POST请求.
     *
     * @param url 请求接口地址
     * @param obj 请求对象，实现了ToJson接口
     * @return 接口响应字符串
     * @throws WxErrorException 异常
     */
    String post(String url, ToJson obj) ;
}
