package cn.nosum.wx.cp.api;

import cn.nosum.wx.cp.entity.WxCpDepart;

import java.util.List;

/**
 * 部门管理接口.
 * 文档地址: https://work.weixin.qq.com/api/doc#90000/90135/
 *
 * @author Young
 */
public interface WxCpDepartmentService {

    /**
     * 部门管理接口 - 创建部门.
     * 最多支持创建500个部门
     *
     * @param depart 部门
     * @return 部门id
     */
    Long create(WxCpDepart depart);

    /**
     * 部门管理接口 - 获取部门列表.
     *
     * @param id 部门id。获取指定部门及其下的子部门。非必需，可为null
     * @return 获取的部门列表
     */
    List<WxCpDepart> list(Long id);

    /**
     * 部门管理接口 - 更新部门.
     * 如果id为0(未部门),1(黑名单),2(星标组)，或者不存在的id，微信会返回系统繁忙的错误
     *
     * @param group 要更新的group，group的id,name必须设置
     */
    void update(WxCpDepart group);

    /**
     * 部门管理接口 - 删除部门.
     * 应用须拥有指定部门的管理权限
     *
     * @param departId 部门id
     */
    void delete(Long departId);
}
