package cn.nosum.wx.cp.api;


import cn.nosum.wx.cp.entity.oa.calendar.WxCpOaCalendar;

import java.util.List;

/**
 * 企业微信日历接口.
 *
 * @author Young
 */
public interface WxCpOaCalendarService {
    /**
     * 创建日历.
     * <pre>
     * 该接口用于通过应用在企业内创建一个日历。
     * 注: 企业微信需要更新到3.0.2及以上版本
     * 请求方式： POST（HTTPS）
     * 请求地址： https://qyapi.weixin.qq.com/cgi-bin/oa/calendar/add?access_token=ACCESS_TOKEN
     *
     * 文档地址：https://work.weixin.qq.com/api/doc/90000/90135/92618
     * </pre>
     *
     * @param calendar 日历对象
     * @return 日历ID
     */
    String add(WxCpOaCalendar calendar);

    /**
     * 更新日历.
     * <pre>
     * 该接口用于修改指定日历的信息。
     * 注意，更新操作是覆盖式，而不是增量式
     * 企业微信需要更新到3.0.2及以上版本
     * 请求方式： POST（HTTPS）
     * 请求地址： https://qyapi.weixin.qq.com/cgi-bin/oa/calendar/update?access_token=ACCESS_TOKEN
     *
     * 文档地址：https://work.weixin.qq.com/api/doc/90000/90135/92619
     * </pre>
     *
     * @param calendar 日历对象
     */
    void update(WxCpOaCalendar calendar);

    /**
     * 获取日历.
     * <pre>
     * 该接口用于获取应用在企业内创建的日历信息。
     *
     * 注: 企业微信需要更新到3.0.2及以上版本
     *
     * 请求方式： POST（HTTPS）
     * 请求地址： https://qyapi.weixin.qq.com/cgi-bin/oa/calendar/get?access_token=ACCESS_TOKEN
     *
     * 文档地址：https://work.weixin.qq.com/api/doc/90000/90135/92621
     * </pre>
     *
     * @param calIds 日历id列表
     * @return 日历对象列表
     */
    List<WxCpOaCalendar> get(List<String> calIds);

    /**
     * 删除日历.
     * <pre>
     * 该接口用于删除指定日历。
     * 注: 企业微信需要更新到3.0.2及以上版本
     * 请求方式： POST（HTTPS）
     * 请求地址： https://qyapi.weixin.qq.com/cgi-bin/oa/calendar/del?access_token=ACCESS_TOKEN
     *
     * 文档地址：https://work.weixin.qq.com/api/doc/90000/90135/92620
     * </pre>
     *
     * @param calId 日历id
     */
    void delete(String calId);
}
