package cn.nosum.wx.cp.api;

import cn.nosum.wx.cp.entity.WxCpTag;
import cn.nosum.wx.cp.entity.WxCpTagAddOrRemoveUsersResult;
import cn.nosum.wx.cp.entity.WxCpTagGetResult;
import cn.nosum.wx.cp.entity.WxCpUser;

import java.util.List;

/**
 * 标签管理接口.
 *
 * @author Young
 */
public interface WxCpTagService {
    /**
     * 创建标签.
     * <pre>
     * 请求地址：https://qyapi.weixin.qq.com/cgi-bin/tag/create?access_token=ACCESS_TOKEN
     * 文档地址：https://work.weixin.qq.com/api/doc#90000/90135/90210
     * </pre>
     *
     * @param name 标签名称，长度限制为32个字以内（汉字或英文字母），标签名不可与其他标签重名。
     * @param id   标签id，非负整型，指定此参数时新增的标签会生成对应的标签id，不指定时则以目前最大的id自增。
     * @return 标签id
     */
    String create(String name, Integer id) ;

    /**
     * 更新标签.
     *
     * @param tagId   标签id
     * @param tagName 标签名
     */
    void update(String tagId, String tagName) ;

    /**
     * 删除标签.
     *
     * @param tagId 标签id
     */
    void delete(String tagId) ;

    /**
     * 获得标签列表.
     *
     * @return 标签列表
     */
    List<WxCpTag> listAll() ;

    /**
     * 获取标签成员.
     *
     * @param tagId 标签ID
     * @return 成员列表
     */
    List<WxCpUser> listUsersByTagId(String tagId) ;

    /**
     * 获取标签成员.
     * 对应: http://qydev.weixin.qq.com/wiki/index.php?title=管理标签 中的get接口
     *
     * @param tagId 标签id
     * @return 标签成员.
     */
    WxCpTagGetResult get(String tagId) ;

    /**
     * 增加标签成员.
     *
     * @param tagId    标签id
     * @param userIds  用户ID 列表
     * @param partyIds 企业部门ID列表
     * @return 增加标签成员结果.
     */
    WxCpTagAddOrRemoveUsersResult addUsers2Tag(String tagId, List<String> userIds, List<String> partyIds) ;

    /**
     * 移除标签成员.
     *
     * @param tagId    标签id
     * @param userIds  用户id列表
     * @param partyIds 企业部门ID列表
     * @return 移除标签成员结果
     */
    WxCpTagAddOrRemoveUsersResult removeUsersFromTag(String tagId, List<String> userIds, List<String> partyIds) ;

}
