package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.error.WxRuntimeException;
import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpAgentService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.WxCpAgent;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;
import lombok.RequiredArgsConstructor;

import java.util.List;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Agent.*;

/**
 * 管理企业号应用.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpAgentServiceImpl implements WxCpAgentService {


    private final WxCpService mainService;

    @Override
    public WxCpAgent get(Integer agentId) {
        if (agentId == null) {
            throw new IllegalArgumentException("缺少agentid参数");
        }

        final String url = String.format(this.mainService.getWxCpConfigStorage().getApiUrl(AGENT_GET), agentId);
        return WxCpAgent.fromJson(this.mainService.get(url, null));
    }

    @Override
    public void set(WxCpAgent agentInfo) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(AGENT_SET);
        String responseContent = this.mainService.post(url, agentInfo.toJson());
        JsonObject jsonObject = GsonParser.parse(responseContent);
        if (jsonObject.get("errcode").getAsInt() != 0) {
            throw new WxRuntimeException(responseContent);
        }
    }

    @Override
    public List<WxCpAgent> list() {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(AGENT_LIST);
        String responseContent = this.mainService.get(url, null);
        JsonObject jsonObject = GsonParser.parse(responseContent);
        if (jsonObject.get("errcode").getAsInt() != 0) {
            throw new WxRuntimeException(responseContent);
        }

        return WxCpGsonBuilder.create().fromJson(jsonObject.get("agentlist").toString(), new TypeToken<List<WxCpAgent>>() {
        }.getType());
    }

}