package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.common.utils.json.WxGsonBuilder;
import cn.nosum.wx.cp.api.WxCpChatService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.WxCpChat;
import cn.nosum.wx.cp.entity.message.WxCpAppChatMessage;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Chat.*;

/**
 * 群聊服务实现.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpChatServiceImpl implements WxCpChatService {
    private final WxCpService cpService;

    @Override
    public String create(String name, String owner, List<String> users, String chatId) {
        Map<String, Object> data = new HashMap<>(4);
        if (StringUtils.isNotBlank(name)) {
            data.put("name", name);
        }
        if (StringUtils.isNotBlank(owner)) {
            data.put("owner", owner);
        }
        if (users != null) {
            data.put("userlist", users);
        }
        if (StringUtils.isNotBlank(chatId)) {
            data.put("chatid", chatId);
        }
        final String url = this.cpService.getWxCpConfigStorage().getApiUrl(APP_CHAT_CREATE);
        String result = this.cpService.post(url, WxGsonBuilder.create().toJson(data));
        return GsonParser.parse(result).get("chatid").getAsString();
    }

    @Override
    public void update(String chatId, String name, String owner, List<String> usersToAdd, List<String> usersToDelete) {
        Map<String, Object> data = new HashMap<>(5);
        if (StringUtils.isNotBlank(chatId)) {
            data.put("chatid", chatId);
        }
        if (StringUtils.isNotBlank(name)) {
            data.put("name", name);
        }
        if (StringUtils.isNotBlank(owner)) {
            data.put("owner", owner);
        }
        if (usersToAdd != null && !usersToAdd.isEmpty()) {
            data.put("add_user_list", usersToAdd);
        }
        if (usersToDelete != null && !usersToDelete.isEmpty()) {
            data.put("del_user_list", usersToDelete);
        }

        final String url = this.cpService.getWxCpConfigStorage().getApiUrl(APP_CHAT_UPDATE);
        this.cpService.post(url, WxGsonBuilder.create().toJson(data));
    }

    @Override
    public WxCpChat get(String chatId) {
        final String url = this.cpService.getWxCpConfigStorage().getApiUrl(APP_CHAT_GET_CHAT_ID + chatId);
        String result = this.cpService.get(url, null);
        final String chatInfo = GsonParser.parse(result).getAsJsonObject("chat_info").toString();
        return WxCpGsonBuilder.create().fromJson(chatInfo, WxCpChat.class);
    }

    @Override
    public void sendMsg(WxCpAppChatMessage message) {
        this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(APP_CHAT_SEND), message.toJson());
    }

}