package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.utils.json.GsonHelper;
import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpDepartmentService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.WxCpDepart;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;
import lombok.RequiredArgsConstructor;

import java.util.List;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Department.*;

/**
 * 部门管理服务实现.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpDepartmentServiceImpl implements WxCpDepartmentService {

    private final WxCpService mainService;

    @Override
    public Long create(WxCpDepart depart) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(DEPARTMENT_CREATE);
        String responseContent = this.mainService.post(url, depart.toJson());
        JsonObject tmpJsonObject = GsonParser.parse(responseContent);
        return GsonHelper.getAsLong(tmpJsonObject.get("id"));
    }

    @Override
    public void update(WxCpDepart group) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(DEPARTMENT_UPDATE);
        this.mainService.post(url, group.toJson());
    }

    @Override
    public void delete(Long departId) {
        String url = String.format(this.mainService.getWxCpConfigStorage().getApiUrl(DEPARTMENT_DELETE), departId);
        this.mainService.get(url, null);
    }

    @Override
    public List<WxCpDepart> list(Long id) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(DEPARTMENT_LIST);
        if (id != null) {
            url += "?id=" + id;
        }

        String responseContent = this.mainService.get(url, null);
        JsonObject tmpJsonObject = GsonParser.parse(responseContent);
        return WxCpGsonBuilder.create()
                .fromJson(tmpJsonObject.get("department"),
                        new TypeToken<List<WxCpDepart>>() {
                        }.getType()
                );
    }
}