package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.error.WxErrorException;
import cn.nosum.wx.common.error.WxRuntimeException;
import cn.nosum.wx.cp.api.WxCpGroupRobotService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import cn.nosum.wx.cp.constant.WxCpApiPathConsts;
import cn.nosum.wx.cp.entity.article.NewArticle;
import cn.nosum.wx.cp.entity.message.WxCpGroupRobotMessage;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

import static cn.nosum.wx.cp.constant.WxCpConstants.GroupRobotMsgType;
import static cn.nosum.wx.cp.constant.WxCpConstants.GroupRobotMsgType.MARKDOWN;
import static cn.nosum.wx.cp.constant.WxCpConstants.GroupRobotMsgType.TEXT;

/**
 * 企业微信群机器人消息发送api 实现.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpGroupRobotServiceImpl implements WxCpGroupRobotService {

    private final WxCpService cpService;

    private String getWebhookUrl() {
        WxCpConfigStorage wxCpConfigStorage = this.cpService.getWxCpConfigStorage();
        final String webhookKey = wxCpConfigStorage.getWebhookKey();
        if (StringUtils.isEmpty(webhookKey)) {
            throw new WxRuntimeException("请先设置WebhookKey");
        }
        return wxCpConfigStorage.getApiUrl(WxCpApiPathConsts.WEBHOOK_SEND) + webhookKey;
    }

    @Override
    public void sendText(String content, List<String> mentionedList, List<String> mobileList) {
        this.sendText(this.getWebhookUrl(), content, mentionedList, mobileList);
    }

    @Override
    public void sendMarkdown(String content) {
        this.sendMarkdown(this.getWebhookUrl(), content);
    }

    @Override
    public void sendImage(String base64, String md5) {
        this.sendImage(this.getWebhookUrl(), base64, md5);
    }

    @Override
    public void sendNews(List<NewArticle> articleList) {
        this.sendNews(this.getWebhookUrl(), articleList);
    }

    @Override
    public void sendText(String webhookUrl, String content, List<String> mentionedList, List<String> mobileList) {
        try {
            this.cpService.postWithoutToken(webhookUrl, new WxCpGroupRobotMessage()
                    .setMsgType(TEXT)
                    .setContent(content)
                    .setMentionedList(mentionedList)
                    .setMentionedMobileList(mobileList)
                    .toJson());
        } catch (WxErrorException e) {
            throw new WxRuntimeException(e.getMessage());
        }
    }

    @Override
    public void sendMarkdown(String webhookUrl, String content) {
        try {
            this.cpService.postWithoutToken(webhookUrl, new WxCpGroupRobotMessage().setMsgType(MARKDOWN).setContent(content).toJson());
        } catch (WxErrorException e) {
            throw new WxRuntimeException(e.getMessage());
        }
    }

    @Override
    public void sendImage(String webhookUrl, String base64, String md5) {
        try {
            this.cpService.postWithoutToken(webhookUrl, new WxCpGroupRobotMessage().setMsgType(GroupRobotMsgType.IMAGE).setBase64(base64).setMd5(md5).toJson());
        } catch (WxErrorException e) {
            throw new WxRuntimeException(e.getMessage());
        }
    }

    @Override
    public void sendNews(String webhookUrl, List<NewArticle> articleList) {
        try {
            this.cpService.postWithoutToken(webhookUrl, new WxCpGroupRobotMessage().setMsgType(GroupRobotMsgType.NEWS).setArticles(articleList).toJson());
        } catch (WxErrorException e) {
            throw new WxRuntimeException(e.getMessage());
        }
    }

}
