package cn.nosum.wx.cp.api.impl;

import cn.nosum.http.utils.UriUtil;
import cn.nosum.wx.common.utils.json.GsonHelper;
import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpOAuth2Service;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.WxCpOauth2UserInfo;
import cn.nosum.wx.cp.entity.WxCpUserDetail;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.gson.JsonObject;
import lombok.RequiredArgsConstructor;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.OAuth2.*;
import static cn.nosum.wx.common.api.WxConsts.OAuth2Scope.*;

/**
 * oauth2 接口实现类.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpOAuth2ServiceImpl implements WxCpOAuth2Service {

    private final WxCpService mainService;

    @Override
    public String buildAuthorizationUrl(String state) {
        return this.buildAuthorizationUrl(
                this.mainService.getWxCpConfigStorage().getOauth2redirectUri(),
                state
        );
    }

    @Override
    public String buildAuthorizationUrl(String redirectUri, String state) {
        return this.buildAuthorizationUrl(redirectUri, state, SNSAPI_BASE);
    }

    @Override
    public String buildAuthorizationUrl(String redirectUri, String state, String scope) {
        StringBuilder url = new StringBuilder(URL_OAUTH2_AUTHORIZE);
        url.append("?appid=").append(this.mainService.getWxCpConfigStorage().getCorpId());
        url.append("&redirect_uri=").append(UriUtil.encodeUriComponent(redirectUri));
        url.append("&response_type=code");
        url.append("&scope=").append(scope);

        if (SNSAPI_PRIVATEINFO.equals(scope) || SNSAPI_USERINFO.equals(scope)) {
            url.append("&agentid=").append(this.mainService.getWxCpConfigStorage().getAgentId());
        }

        if (state != null) {
            url.append("&state=").append(state);
        }

        url.append("#wechat_redirect");
        return url.toString();
    }

    @Override
    public WxCpOauth2UserInfo getUserInfo(String code) {
        return this.getUserInfo(this.mainService.getWxCpConfigStorage().getAgentId(), code);
    }

    @Override
    public WxCpOauth2UserInfo getUserInfo(Integer agentId, String code) {
        String responseText = this.mainService.get(String.format(this.mainService.getWxCpConfigStorage().getApiUrl(GET_USER_INFO), code, agentId), null);
        JsonObject jo = GsonParser.parse(responseText);

        return WxCpOauth2UserInfo.builder()
                .userId(GsonHelper.getString(jo, "UserId"))
                .deviceId(GsonHelper.getString(jo, "DeviceId"))
                .openId(GsonHelper.getString(jo, "OpenId"))
                .userTicket(GsonHelper.getString(jo, "user_ticket"))
                .expiresIn(GsonHelper.getString(jo, "expires_in"))
                .externalUserId(GsonHelper.getString(jo, "external_userid"))
                .build();
    }

    @Override
    public WxCpUserDetail getUserDetail(String userTicket) {
        JsonObject param = new JsonObject();
        param.addProperty("user_ticket", userTicket);
        String responseText = this.mainService.post(this.mainService.getWxCpConfigStorage().getApiUrl(GET_USER_DETAIL), param.toString());
        return WxCpGsonBuilder.create().fromJson(responseText, WxCpUserDetail.class);
    }
}