package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.utils.json.GsonHelper;
import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpOaCalendarService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.oa.calendar.WxCpOaCalendar;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.gson.reflect.TypeToken;
import lombok.RequiredArgsConstructor;

import java.util.List;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Oa.*;

/**
 * 企业微信日历实现.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpOaCalendarServiceImpl implements WxCpOaCalendarService {
    private final WxCpService wxCpService;

    @Override
    public String add(WxCpOaCalendar calendar) {
        return this.wxCpService.post(this.wxCpService.getWxCpConfigStorage().getApiUrl(CALENDAR_ADD), calendar);
    }

    @Override
    public void update(WxCpOaCalendar calendar) {
        this.wxCpService.post(this.wxCpService.getWxCpConfigStorage().getApiUrl(CALENDAR_UPDATE), calendar);
    }

    @Override
    public List<WxCpOaCalendar> get(List<String> calIds) {
        String response = this.wxCpService.post(this.wxCpService.getWxCpConfigStorage().getApiUrl(CALENDAR_GET),
                GsonHelper.buildJsonObject("cal_id_list", calIds));
        return WxCpGsonBuilder.create().fromJson(GsonParser.parse(response).get("calendar_list").getAsJsonArray().toString(),
                new TypeToken<List<WxCpOaCalendar>>() {
                }.getType());
    }

    @Override
    public void delete(String calId) {
        this.wxCpService.post(this.wxCpService.getWxCpConfigStorage().getApiUrl(CALENDAR_DEL), GsonHelper.buildJsonObject("cal_id", calId));
    }
}
