package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpOaScheduleService;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.entity.oa.WxCpOaSchedule;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.common.collect.ImmutableMap;
import com.google.gson.reflect.TypeToken;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Oa.*;

/**
 * 企业微信日程接口实现类.
 *
 * @author Young
 */
@Slf4j
@RequiredArgsConstructor
public class WxCpOaOaScheduleServiceImpl implements WxCpOaScheduleService {
    private final WxCpService cpService;

    @Override
    public String add(WxCpOaSchedule schedule, Integer agentId) {
        Map<String, Serializable> param;
        if (agentId == null) {
            param = ImmutableMap.of("schedule", schedule);
        } else {
            param = ImmutableMap.of("schedule", schedule, "agentid", agentId);
        }

        return this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(SCHEDULE_ADD),
                WxCpGsonBuilder.create().toJson(param));
    }

    @Override
    public void update(WxCpOaSchedule schedule) {
        this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(SCHEDULE_UPDATE),
                WxCpGsonBuilder.create().toJson(ImmutableMap.of("schedule", schedule)));
    }

    @Override
    public List<WxCpOaSchedule> getDetails(List<String> scheduleIds) {
        final String response = this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(SCHEDULE_GET),
                WxCpGsonBuilder.create().toJson(ImmutableMap.of("schedule_id_list", scheduleIds)));
        return WxCpGsonBuilder.create().fromJson(GsonParser.parse(response).get("schedule_list"),
                new TypeToken<List<WxCpOaSchedule>>() {
                }.getType());
    }

    @Override
    public void delete(String scheduleId) {
        this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(SCHEDULE_DEL),
                WxCpGsonBuilder.create().toJson(ImmutableMap.of("schedule_id", scheduleId)));
    }

    @Override
    public List<WxCpOaSchedule> listByCalendar(String calId, Integer offset, Integer limit) {
        final Map<String, Object> param = new HashMap<>(3);
        param.put("cal_id", calId);
        if (offset != null) {
            param.put("offset", offset);
        }
        if (limit != null) {
            param.put("limit", limit);
        }
        final String response = this.cpService.post(this.cpService.getWxCpConfigStorage().getApiUrl(SCHEDULE_LIST),
                WxCpGsonBuilder.create().toJson(param));
        return WxCpGsonBuilder.create().fromJson(GsonParser.parse(response).get("schedule_list"),
                new TypeToken<List<WxCpOaSchedule>>() {
                }.getType());
    }
}