package cn.nosum.wx.cp.api.impl;


import cn.nosum.http.RequestHttpProperties;
import cn.nosum.http.apache.ApacheHttpClientBuilder;
import cn.nosum.http.apache.DefaultApacheHttpClientBuilder;
import cn.nosum.http.enums.HttpType;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import org.apache.http.HttpHost;
import org.apache.http.impl.client.CloseableHttpClient;

/**
 * 企业微信API服务层APACHE实现.
 *
 * @author Young
 */
public class WxCpServiceApacheHttpClientImpl extends BaseWxCpServiceImpl<CloseableHttpClient, HttpHost> {
    private CloseableHttpClient httpClient;
    private HttpHost httpProxy;
    private ApacheHttpClientBuilder apacheHttpClientBuilder;

    @Override
    public void initHttp() {
        ApacheHttpClientBuilder apacheHttpClientBuilder = getApacheHttpClientBuilder();
        if (null == apacheHttpClientBuilder) {
            apacheHttpClientBuilder = DefaultApacheHttpClientBuilder.get();
        }

        RequestHttpProperties httpProperties = this.configStorage.getHttpProperties();
        apacheHttpClientBuilder.httpProxyHost(httpProperties.getHttpProxyHost())
                .httpProxyPort(httpProperties.getHttpProxyPort())
                .httpProxyUsername(httpProperties.getHttpProxyUsername())
                .httpProxyPassword(httpProperties.getHttpProxyPassword())
                .connectTimeout(httpProperties.getConnectTimeout())
                .readTimeout(httpProperties.getReadTimeout());

        if (httpProperties.getHttpProxyHost() != null && httpProperties.getHttpProxyPort() > 0) {
            this.httpProxy = new HttpHost(httpProperties.getHttpProxyHost(), httpProperties.getHttpProxyPort());
        }

        this.httpClient = apacheHttpClientBuilder.build();
    }

    @Override
    public CloseableHttpClient getRequestHttpClient() {
        return httpClient;
    }

    @Override
    public HttpHost getRequestHttpProxy() {
        return httpProxy;
    }

    @Override
    public HttpType getRequestType() {
        return HttpType.APACHE_HTTP;
    }

    public void setApacheHttpClientBuilder(ApacheHttpClientBuilder apacheHttpClientBuilder) {
        this.apacheHttpClientBuilder = apacheHttpClientBuilder;
    }

    public ApacheHttpClientBuilder getApacheHttpClientBuilder() {
        return apacheHttpClientBuilder;
    }

    @Override
    public WxCpConfigStorage getWxCpConfigStorage() {
        return this.configStorage;
    }

}
