package cn.nosum.wx.cp.api.impl;

import cn.nosum.http.RequestHttpProperties;
import cn.nosum.http.enums.HttpType;
import cn.nosum.http.okhttp.OkHttpProxyInfo;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Credentials;
import okhttp3.OkHttpClient;

import java.util.concurrent.TimeUnit;

/**
 * 企业微信API服务层 OKHTTP3 实现.
 *
 * @author Young
 */
@Slf4j
public class WxCpServiceOkHttpImpl extends BaseWxCpServiceImpl<OkHttpClient, OkHttpProxyInfo> {

    private OkHttpClient httpClient;
    private OkHttpProxyInfo httpProxy;


    @Override
    public void initHttp() {
        RequestHttpProperties httpProperties = this.configStorage.getHttpProperties();
        if (httpProperties.getHttpProxyHost() != null && httpProperties.getHttpProxyPort() > 0) {
            httpProxy = OkHttpProxyInfo.httpProxy(httpProperties.getHttpProxyHost(),
                    httpProperties.getHttpProxyPort(),
                    httpProperties.getHttpProxyUsername(),
                    httpProperties.getHttpProxyPassword());
        }

        OkHttpClient.Builder clientBuilder = new OkHttpClient.Builder();
        if (httpProxy != null) {
            clientBuilder.proxy(getRequestHttpProxy().getProxy());
            clientBuilder.authenticator((route, response) -> {
                String credential = Credentials.basic(httpProxy.getProxyUsername(), httpProxy.getProxyPassword());
                return response.request().newBuilder()
                        .header("Authorization", credential)
                        .build();
            });
        }

        clientBuilder
                .connectTimeout(httpProperties.getConnectTimeout(), TimeUnit.MILLISECONDS)
                .readTimeout(httpProperties.getReadTimeout(),TimeUnit.MILLISECONDS);

        httpClient = clientBuilder.build();
    }

    @Override
    public OkHttpClient getRequestHttpClient() {
        return httpClient;
    }

    @Override
    public OkHttpProxyInfo getRequestHttpProxy() {
        return httpProxy;
    }

    @Override
    public HttpType getRequestType() {
        return HttpType.OK_HTTP;
    }

    @Override
    public WxCpConfigStorage getWxCpConfigStorage() {
        return this.configStorage;
    }
}
