package cn.nosum.wx.cp.api.impl;

import cn.nosum.wx.common.utils.json.GsonParser;
import cn.nosum.wx.cp.api.WxCpService;
import cn.nosum.wx.cp.api.WxCpTagService;
import cn.nosum.wx.cp.entity.WxCpTag;
import cn.nosum.wx.cp.entity.WxCpTagAddOrRemoveUsersResult;
import cn.nosum.wx.cp.entity.WxCpTagGetResult;
import cn.nosum.wx.cp.entity.WxCpUser;
import cn.nosum.wx.cp.utils.json.WxCpGsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.gson.reflect.TypeToken;
import lombok.RequiredArgsConstructor;

import java.util.List;

import static cn.nosum.wx.cp.constant.WxCpApiPathConsts.Tag.*;

/**
 * 标签管理接口.
 *
 * @author Young
 */
@RequiredArgsConstructor
public class WxCpTagServiceImpl implements WxCpTagService {

    private final WxCpService mainService;

    @Override
    public String create(String name, Integer id) {
        JsonObject o = new JsonObject();
        o.addProperty("tagname", name);

        if (id != null) {
            o.addProperty("tagid", id);
        }
        return this.create(o);
    }

    private String create(JsonObject param) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(TAG_CREATE);
        String responseContent = this.mainService.post(url, param.toString());
        JsonObject jsonObject = GsonParser.parse(responseContent);
        return jsonObject.get("tagid").getAsString();
    }

    @Override
    public void update(String tagId, String tagName) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(TAG_UPDATE);
        JsonObject o = new JsonObject();
        o.addProperty("tagid", tagId);
        o.addProperty("tagname", tagName);
        this.mainService.post(url, o.toString());
    }

    @Override
    public void delete(String tagId) {
        String url = String.format(this.mainService.getWxCpConfigStorage().getApiUrl(TAG_DELETE), tagId);
        this.mainService.get(url, null);
    }

    @Override
    public List<WxCpTag> listAll() {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(TAG_LIST);
        String responseContent = this.mainService.get(url, null);
        JsonObject tmpJson = GsonParser.parse(responseContent);
        return WxCpGsonBuilder.create()
                .fromJson(
                        tmpJson.get("taglist"),
                        new TypeToken<List<WxCpTag>>() {
                        }.getType()
                );
    }

    @Override
    public List<WxCpUser> listUsersByTagId(String tagId) {
        String url = String.format(this.mainService.getWxCpConfigStorage().getApiUrl(TAG_GET), tagId);
        String responseContent = this.mainService.get(url, null);
        JsonObject tmpJson = GsonParser.parse(responseContent);
        return WxCpGsonBuilder.create()
                .fromJson(
                        tmpJson.get("userlist"),
                        new TypeToken<List<WxCpUser>>() {
                        }.getType()
                );
    }

    @Override
    public WxCpTagAddOrRemoveUsersResult addUsers2Tag(String tagId, List<String> userIds, List<String> partyIds) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(TAG_ADD_TAG_USERS);
        JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("tagid", tagId);
        this.addUserIdsAndPartyIdsToJson(userIds, partyIds, jsonObject);

        return WxCpTagAddOrRemoveUsersResult.fromJson(this.mainService.post(url, jsonObject.toString()));
    }

    @Override
    public WxCpTagAddOrRemoveUsersResult removeUsersFromTag(String tagId, List<String> userIds, List<String> partyIds) {
        String url = this.mainService.getWxCpConfigStorage().getApiUrl(TAG_DEL_TAG_USERS);
        JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("tagid", tagId);
        this.addUserIdsAndPartyIdsToJson(userIds, partyIds, jsonObject);

        return WxCpTagAddOrRemoveUsersResult.fromJson(this.mainService.post(url, jsonObject.toString()));
    }

    private void addUserIdsAndPartyIdsToJson(List<String> userIds, List<String> partyIds, JsonObject jsonObject) {
        if (userIds != null) {
            JsonArray jsonArray = new JsonArray();
            for (String userId : userIds) {
                jsonArray.add(new JsonPrimitive(userId));
            }
            jsonObject.add("userlist", jsonArray);
        }

        if (partyIds != null) {
            JsonArray jsonArray = new JsonArray();
            for (String userId : partyIds) {
                jsonArray.add(new JsonPrimitive(userId));
            }
            jsonObject.add("partylist", jsonArray);
        }
    }

    @Override
    public WxCpTagGetResult get(String tagId) {
        if (tagId == null) {
            throw new IllegalArgumentException("缺少tagId参数");
        }

        String url = String.format(this.mainService.getWxCpConfigStorage().getApiUrl(TAG_GET), tagId);
        String responseContent = this.mainService.get(url, null);
        return WxCpTagGetResult.fromJson(responseContent);
    }
}
