package cn.nosum.wx.cp.entity.message;

import cn.nosum.wx.cp.utils.crypto.WxCpCryptUtil;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import lombok.Data;
import cn.nosum.wx.common.utils.xml.XStreamCDataConverter;
import cn.nosum.wx.cp.entity.outxmlbuilder.*;
import cn.nosum.wx.cp.config.WxCpConfigStorage;
import cn.nosum.wx.cp.utils.xml.StreamXmlTransformer;

import java.io.Serializable;

/**
 * 被动回复消息.
 * https://work.weixin.qq.com/api/doc#12975
 *
 * @author Young
 */
@XStreamAlias("xml")
@Data
public abstract class WxCpXmlOutMessage implements Serializable {
    private static final long serialVersionUID = 1418629839964153110L;

    @XStreamAlias("ToUserName")
    @XStreamConverter(value = XStreamCDataConverter.class)
    protected String toUserName;

    @XStreamAlias("FromUserName")
    @XStreamConverter(value = XStreamCDataConverter.class)
    protected String fromUserName;

    @XStreamAlias("CreateTime")
    protected Long createTime;

    @XStreamAlias("MsgType")
    @XStreamConverter(value = XStreamCDataConverter.class)
    protected String msgType;

    /**
     * 获得文本消息builder.
     */
    public static TextBuilder TEXT() {
        return new TextBuilder();
    }

    /**
     * 获得图片消息builder.
     */
    public static ImageBuilder IMAGE() {
        return new ImageBuilder();
    }

    /**
     * 获得语音消息builder.
     */
    public static VoiceBuilder VOICE() {
        return new VoiceBuilder();
    }

    /**
     * 获得视频消息builder.
     */
    public static VideoBuilder VIDEO() {
        return new VideoBuilder();
    }

    /**
     * 获得图文消息builder.
     */
    public static NewsBuilder NEWS() {
        return new NewsBuilder();
    }

    /**
     * 获得任务卡片消息builder.
     */
    public static TaskCardBuilder TASK_CARD() {
        return new TaskCardBuilder();
    }

    protected String toXml() {
        return StreamXmlTransformer.toXml((Class) this.getClass(), this);
    }

    /**
     * 转换成加密的xml格式.
     */
    public String toEncryptedXml(WxCpConfigStorage wxCpConfigStorage) {
        String plainXml = toXml();
        return new WxCpCryptUtil(wxCpConfigStorage).encrypt(plainXml);
    }
}
